@@ -1,14 +1,12 @@
-#! /usr/local/bin/perl
+#! /usr/bin/perl
 
-#makes things work when run without install
-use lib qw( ../perl-shared/blib/lib ../perl-shared/blib/arch );
-# this is for after install
-use lib qw( /usr/local/lib/perl ../lib/perl );
+use lib qw( /usr/local/lib/perl );
 
 use RRDs;
 my $start=time;
 my $rrd="randome.rrd";
 my $name = $0;
+$name =~ s/.*\///g;
 $name =~ s/\.pl.*//g;
 
 RRDs::create ($rrd, "--start",$start-1, "--step",300,
@@ -46,9 +44,9 @@ RRDs::graph "$name.png",
 ;
 
 if ($ERROR = RRDs::error) {
-  print "ERROR: $ERROR\n";
+  die "ERROR: $ERROR\n";
 };
 
 
 print "This script has created $name.png in the current directory\n";
-print "This demonstrates the use of the TIME and % RPN operators\n";
+print "This demonstrates the use of MIN and MAX archives\n";
